macroScript Adjust_Pivot
	category: "AKNToolkit"
	toolTip: "Adjust Pivot to min/max or parent"
	buttonText: "Adjust Pivot"
	icon: #("AKNToolkit",2)
(


	local icon24i = (pathConfig.GetDir #userScripts)+"\\AKNToolkit\\icons\\adjust_pivot_24i.bmp";
	local icon24a = (pathConfig.GetDir #userScripts)+"\\AKNToolkit\\icons\\adjust_pivot_24a.bmp";
	
	fn setPivotOnChildren item mainPivot = (
		if item.children != undefined do	(
			-- Set all children's pivot to match top level parent
			item.children.pivot = mainPivot
			for obj in item.children do (
				setPivotOnChildren obj mainPivot
			)
		)
	)
	fn set_pivot setX setY setZ = (	
		-- Set the setX/setY/setZ  axles according to these values: -1 = min size, 0 = center, 1 = max size, null = ignore
		local selected_objects = for items in getCurrentSelection() where isValidNode items collect items
		for obj in selected_objects do (
			local obj_pivot = obj.pivot

			if setX == -1 then (
				obj_pivot[1] = obj.min.x
			) else if setX == 0 then (
				obj_pivot[1] = obj.min.x + ((obj.max.x-obj.min.x)/2)
			) else if setX == 1 then (
				obj_pivot[1] = obj.max.x
			)
			
			if setY == -1 then (
				obj_pivot[2] = obj.min.y
			) else if setY == 0 then (
				obj_pivot[2] = obj.min.y + ((obj.max.y-obj.min.y)/2)
			) else if setY == 1 then (
				obj_pivot[2] = obj.max.y
			)
			
			if setZ == -1 then (
				obj_pivot[3] = obj.min.z
			) else if setZ == 0 then (
				obj_pivot[3] = obj.min.z + ((obj.max.z-obj.min.z)/2)
			) else if setZ == 1 then (
				obj_pivot[3] = obj.max.z
			)
			format "New Pivot: %\n" obj_pivot
			
			obj.pivot = obj_pivot
		)
		
	)
	
	fn set_pivot_to_zero = (
		local selected_objects = for items in getCurrentSelection() where isValidNode items collect items
		selected_objects.pivot = [0,0,0]
	)
	
	rollout adjustpivotRollout "Adjust Pivot" width:132 height:168
	(
		groupBox 'grp_Pivot2BB' "Pivot to Bounding Box" pos:[3,3] width:126 height:100
		button 'btn_MinXMaxY' "" pos:[10,21] width:24 height:24 toolTip:"Min X, Max Y" images:#(icon24i,icon24a,13,1,1,0,0)
		button 'btn_CenterXMaxY' "" pos:[36,21] width:24 height:24 toolTip:"Center X, Max Y" images:#(icon24i,icon24a,13,2,2,0,0)
		button 'btn_MaxXMaxY' "" pos:[62,21] width:24 height:24 toolTip:"Max X, Max Y" images:#(icon24i,icon24a,13,3,3,0,0)
		button 'btn_MinXCenterY' "" pos:[10,47] width:24 height:24 toolTip:"Min X, Center Y" images:#(icon24i,icon24a,13,4,4,0,0)
		button 'btn_CenterXCenterY' "" pos:[36,47] width:24 height:24 toolTip:"Center X, Center Y" images:#(icon24i,icon24a,13,5,5,0,0)
		button 'btn_MaxXCenterY' "" pos:[62,47] width:24 height:24 toolTip:"Max X, Center Y" images:#(icon24i,icon24a,13,6,6,0,0)
		button 'btn_MinXMinY' "" pos:[10,73] width:24 height:24 toolTip:"Min X, Min Y" images:#(icon24i,icon24a,13,7,7,0,0)
		button 'btn_CenterXMinY' "" pos:[36,73] width:24 height:24 toolTip:"Center X, Min Y" images:#(icon24i,icon24a,13,8,8,0,0)
		button 'btn_MaxXMinY' "" pos:[62,73] width:24 height:24 toolTip:"Max X, Min Y" images:#(icon24i,icon24a,13,9,9,0,0)
		button 'btn_MaxZ' "" pos:[96,21] width:24 height:24 toolTip:"Max Z" images:#(icon24i,icon24a,13,10,10,1,1)
		button 'btn_CenterZ' "" pos:[96,47] width:24 height:24 toolTip:"Center Z" images:#(icon24i,icon24a,13,11,11,1,1)
		button 'btn_MinZ' "" pos:[96,73] width:24 height:24 toolTip:"MinZ" images:#(icon24i,icon24a,13,12,12,1,1)
		GroupBox 'grp_Pivot2Parent' "" pos:[3,99] width:126 height:64 align:#left
		button 'btn_PivotToParent' "Match Pivot to Parent" pos:[9,109] width:114 height:24 align:#left
		button 'btn_PivotToZero' "Set Pivot to 0,0,0" pos:[9,136] width:114 height:24 align:#left
			
		on btn_MinXMinY pressed do (
			set_pivot -1 -1 null
		)
		on btn_CenterXMinY pressed do (
			set_pivot 0 -1 null
		)
		on btn_MaxXMinY pressed do (
			set_pivot 1 -1 null
		)
		on btn_MinXCenterY pressed do (
			set_pivot -1 0 null
		)
		on btn_CenterXCenterY pressed do (
			set_pivot 0 0 null
		)
		on btn_MaxXCenterY pressed do (
			set_pivot 1 0 null
		)
		on btn_MinXMaxY pressed do (
			set_pivot -1 1 null
		)
		on btn_CenterXMaxY pressed do (
			set_pivot 0 1 null
		)
		on btn_MaxXMaxY pressed do (
			set_pivot 1 1 null
		)
		on btn_MaxZ pressed do (
			set_pivot null null 1
		)
		on btn_CenterZ pressed do (
			set_pivot null null 0
		)
		on btn_MinZ pressed do (
			set_pivot null null -1
		)
		
		on btn_PivotToParent pressed do (
			if (queryBox "Do you want to set all selected objects pivots to match the top level parent?" title:"Match pivot to parent") do (
				undo "Children Pivot Match Parent" on (
					-- Get the top level objects from the selected objects
					local main_sets = for items in getCurrentSelection() where isValidNode items AND items.parent == undefined collect items
					for item in main_sets do (
						-- Get top level object pivot
						local mainPivot = item.pivot
						-- Check if the item has children and set pivot if so, and then filter down the level to get all children
						setPivotOnChildren item mainPivot
					)
				)
			)
		)
		on btn_PivotToZero pressed do (
			if (queryBox "Do you want to set all selected objects pivots to 0,0,0?" title:"Set pivot to 0,0,0") do (
				undo "Pivot to 0,0,0" on (
					local selected_objects = for items in getCurrentSelection() where isValidNode items collect items
					selected_objects.pivot = [0,0,0]
				)
			)
		)
	)
	
	
	
	try(destroyDialog adjustpivotRollout)catch()	
	createdialog adjustpivotRollout 132 168 100 100 lockHeight:true
)